/*
 * This file is part of decoding
 * Copyright 2012  Guillaume Quintin, INRIA
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#define _POSIX_C_SOURCE 1

#include <time.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <decoding/decoding.h>

#define RING_NAME fld
#define PRIME 67
#include <decoding/rings/gfp_word.c>
/* #include <decoding/rings/GF5.c> */
#include <decoding/algos.c>

void bench_upol_shift_naive(int d,int n) {
  clock_t t0,t1;
  double tt;
  int i;
  fld *f,*g,b;
  f = fld_ll_upol_malloc_random(d);
  g = fld_ll_vec_malloc_init(d);
  fld_init(b);
  t0 = clock();
  for( i = 0 ; i <= n ; i++ ) {
    fld_random(b);
    fld_ll_upol_shift_naive(g,f,d,b);
  }
  t1 = clock();
  fld_ll_vec_free(f,d);
  fld_ll_vec_free(g,d);
  fld_clear(b);
  tt = (double)(t1 - t0) / (double)CLOCKS_PER_SEC;
  printf("upol_shift_naive: %f\n",tt);
}

void bench_upol_shift(int d,int n) {
  clock_t t0,t1;
  double tt;
  int i;
  fld *f,*g,b;
  f = fld_ll_upol_malloc_random(d);
  g = fld_ll_vec_malloc_init(d);
  fld_init(b);
  t0 = clock();
  for( i = 0 ; i <= n ; i++ ) {
    fld_random(b);
    fld_ll_upol_shift(g,f,d,b);
  }
  t1 = clock();
  fld_ll_vec_free(f,d);
  fld_ll_vec_free(g,d);
  fld_clear(b);
  tt = (double)(t1 - t0) / (double)CLOCKS_PER_SEC;
  printf("upol_shift: %f\n",tt);
}

int main(int argc,char **argv) {
  int d,n;
  if ( argc != 3 ) return -1;
  srand(time(0));
  print_compile_flags(stdout);
  d = atoi(argv[1]);
  n = atoi(argv[2]);
  fld_ring_init();
  bench_upol_shift_naive(d,n);
  bench_upol_shift(d,n);
  return 0;
}

