/*
 * This file is part of decoding
 * Copyright 2012  Guillaume Quintin, Daniel Augot, INRIA
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#include <iostream>
#include <sstream>
#include "rsdecoder.h"
#define NOT_FOUND 1

int member(ZZ_pX mpol,vector< RecoveryPoly< ZZ_pX > > list) {
  bool res = false;
  for(int i = 0 ; i < list.size() ; ++i ) {
    if ( mpol == list[i].phi ) {
      res = true;
      break;
    }
  }
  return res;
}

int benchp(int p,int n,int k,int tau,int n_iter) {
  int mu = n - tau;
  int v = k - 1;

  ZZ zz_modulus; 
  zz_modulus = p;
  ZZ_p::init(zz_modulus);

  ZZ zz_one;
  zz_one = 1;

  RSDecoder< ZZ_p,vec_ZZ_p,ZZ_pX,ZZ_pXY,mat_ZZ_p > decoder(zz_one,zz_modulus);

  vec_ZZ_p support,rsword,yword;
  rsword.SetLength(n);
  yword.SetLength(n);
  support.SetLength(n);

  TestType type = KOTTER;

  vector< unsigned short > correct(n);

  vector< RecoveryPoly< ZZ_pX > > res; 

  for(int i = 0 ; i < n ; ++i ) {
    support[i] = i ;
    correct[i] = i;
  }

  ZZ_pX mpol;

  cout << "   p = " << p << endl;
  cout << "   n = " << n << endl;
  cout << "   k = " << k << endl;
  cout << "   t = " << (n - k) / 2 << endl;
  cout << "  mu = " << mu << endl;
  cout << " tau = " << tau << endl;;
  cout << "supp = " << support << endl;

  for(int j = 0 ; j < n_iter ; ++j ) {
    mpol = random_ZZ_pX(k);
    
    for(int i = 0 ; i < n ; ++i ) {
      eval(rsword[i],mpol,support[i]); 
    }

    for(int i = 0 ; i < n ; ++i) {
      yword[i] = rsword[i];
    }

    for(int i = mu ; i < n ; ++i ) {
      yword[i] = yword[i] + 1;
    }

    /*
    cout << "rsword is \t" << rsword << endl;
    cout << "yword is \t" << yword << endl;
    */
    
    res = decoder.findpolys_gs(v,mu,correct,support,yword,type);

    if ( member(mpol,res) == false ) {
      return NOT_FOUND;
    }
  }
  return 0;
}

int main(int argc,char** argv) {
  if (argc != 6 ) {
    cerr << "usage:" << argv[0] << " p n k tau n_iter" << endl;
    cerr << "where n_iter is the number of decoding attempts" << endl;
    exit(-1);
  }
  int p = atoi(argv[1]);
  int n = atoi(argv[2]);
  int k = atoi(argv[3]);
  int tau = atoi(argv[4]);
  int n_iter = atoi(argv[5]);
  if ( benchp(p,n,k,tau,n_iter) == NOT_FOUND ) {
    cerr << "Polynomial not found in decoding list" << endl;
    return -1;
  }
  return 0;
}

