/*
 * This file is part of decoding
 * Copyright 2012  Guillaume Quintin, Daniel Augot, INRIA
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#include <iostream>
#include <sstream>
#include "rsdecoder.h"
#include <NTL/GF2XFactoring.h>
#define NOT_FOUND 1

int member(GF2EX mpol,vector< RecoveryPoly< GF2EX > > list) {
  bool res = false;
  for( int i = 0 ; i < list.size() ; ++i ) {
    if ( mpol == list[i].phi ) {
      res=true;
      break;
    }
  }
  return res;
}

int benchq(int p,int m,int n,int k,int tau,int n_iter) {
  int mu = n - tau;
  GF2X defpol;

  /* X^8 + X^4 + X^3 + X^2 + 1 */

  SetCoeff(defpol,8,1);
  SetCoeff(defpol,4,1);
  SetCoeff(defpol,3,1);
  SetCoeff(defpol,2,1);
  SetCoeff(defpol,0,1);
  /*BuildSparseIrred(defpol,m);*/
  GF2E::init(defpol);
  // GF2X::HexOutput = 1;
  
  vec_GF2E support;
  vec_GF2E rsword,yword;
  vector< unsigned short > correct(n);
  vector< RecoveryPoly< GF2EX > > res; 
  RSDecoder< GF2E,vec_GF2E,GF2EX,GF2EXY,mat_GF2E > decoder; 
  
  rsword.SetLength(n);
  yword.SetLength(n);
  support.SetLength(n);

  support.SetLength(n);
  for( int i = 0 ; i < n ; ++i ) {
    GF2X pol = GF2X(i,1);
    support[i] = to_GF2E(pol);
    correct[i] = i;
  }

  TestType type = KOTTER;

  GF2EX mpol;

  /*
  cout << "   p = " << p << endl;
  cout << "   n = " << n << endl;
  cout << "   k = " << k << endl;
  cout << "   t = " << (n - k) / 2 << endl;
  cout << "  mu = " << mu << endl;
  cout << " tau = " << tau << endl;;
  cout << "supp = " << support << endl;
  */

  clock_t start = clock();

  for( int j = 0 ; j < n_iter ; ++j ) {
    mpol = random_GF2EX(k);
    for( int i = 0 ; i < n ; ++i ) {
      eval(rsword[i],mpol,support[i]); 
    }

    for( int i = 0 ; i < n ; ++i ) {
      yword[i] = rsword[i];
    }
    for( int i = mu ; i < n ; ++i ) { // errors in last positions.
      yword[i] = yword[i] + 1;
    }

    /*
    cout <<  "rsword is \t" << rsword << endl;
    cout <<  "yword is \t" << yword << endl;
    */

    res = decoder.findpolys_gs(k,mu,correct,support,yword,type);
    if ( member (mpol,res) == false ) {
      return NOT_FOUND;
    }
  }

  clock_t end = clock();
  double tt = (end - start) / CLOCKS_PER_SEC;

  printf("Time: %f\n",tt);

  return 0;
}

int main(int argc,char** argv) {
  if ( argc != 7 ) {
    cerr << "usage:" << argv[0] << " p m  n k tau n_iter" << endl;
    cerr << "where the field is GF(p^m)" << endl;
    cerr << "where n_iter is the number of decoding attempts" << endl;
    exit(-1);
  }
  int p = atoi(argv[1]);
  int m = atoi(argv[2]);
  int n = atoi(argv[3]);
  int k = atoi(argv[4]);
  int tau = atoi(argv[5]);
  int n_iter = atoi(argv[6]);

  if ( benchq(p,m,n,k,tau,n_iter) == NOT_FOUND ) {
    cerr << "Polynomial not found in decoding list" << endl;
    return -1;
  }
  return 0;
}

