/*
 * This file is part of decoding
 * Copyright 2012  Guillaume Quintin, INRIA
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#define _POSIX_C_SOURCE 1

#include <time.h>
#include <stdio.h>
#include <decoding/decoding.h>

#define RING_NAME fld
/* #define PRIME 101 */
#define PRIME 67
#include <decoding/rings/gfp_word.c>
#include <decoding/algos.c>

void bench_ll_upol_mul_naive(int d,int n) {
  clock_t t0,t1;
  double tt;
  int i;
  fld *a,*b,*c;
  a = fld_ll_vec_malloc_init(d);
  b = fld_ll_vec_malloc_init(d);
  c = fld_ll_vec_malloc_init(2 * d);
  fld_ll_vec_random(a,d);
  fld_ll_vec_random(b,d);
  t0 = clock();
  for( i = 0 ; i < n ; i++ ) {
    (void) fld_ll_upol_mul_naive(c,a,d,b,d);
  }
  t1 = clock();
  fld_ll_vec_free(a,d);
  fld_ll_vec_free(b,d);
  fld_ll_vec_free(c,2 * d);
  tt = (double)(t1 - t0) / (double)(CLOCKS_PER_SEC);
  printf("ll_upol_mul_naive: degree %d (x%d): %f\n",d,n,tt);
}

void bench_ll_upol_mul(int d,int n) {
  clock_t t0,t1;
  double tt;
  int i;
  fld *a,*b,*c;
  a = fld_ll_vec_malloc_init(d);
  b = fld_ll_vec_malloc_init(d);
  c = fld_ll_vec_malloc_init(2 * d);
  fld_ll_vec_random(a,d);
  fld_ll_vec_random(b,d);
  t0 = clock();
  for( i = 0 ; i < n ; i++ ) {
    (void) fld_ll_upol_mul(c,a,d,b,d);
  }
  t1 = clock();
  fld_ll_vec_free(a,d);
  fld_ll_vec_free(b,d);
  fld_ll_vec_free(c,2 * d);
  tt = (double)(t1 - t0) / (double)(CLOCKS_PER_SEC);
  printf("ll_upol_mul: degree %d (x%d): %f\n",d,n,tt);
}

int main(int argc,char **argv) {
  int d,n;
  if ( argc != 3 ) return -1;
  d = atoi(argv[1]);
  n = atoi(argv[2]);
  fld_ring_init();
  bench_ll_upol_mul_naive(d,n);
  bench_ll_upol_mul(d,n);
  return 0;
}

