/*
 * This file is part of decoding
 * Copyright 2012  Guillaume Quintin, Barbier Morgan, INRIA
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

/* Berlekamp-Massey algorithm: algorithm 6.1, page 258,
   ERROR CORRECTION CODING Mathematical Methods and Algorithms,
   Todd K. Moon */

void ll_berlekamp_massey(R *r,int *nr,R *a,int na) {
  int k,i,L,l,nbuf,np,nc;
  R *buf,*tmp,*c,*p,*t,d,dm,A;

  R_init(d);
  R_init(dm);
  R_init(A);
  nbuf = 2 * (na + 1);
  buf = ll_vec_malloc_init(nbuf);

  c = r;
  p = buf;
  t = &buf[na + 1];
  R_castint(c[0],1);
  R_castint(p[0],1);
  L = 0;
  nc = 1;
  l = 1;
  np = 1;
  R_castint(dm,1);
  for( k = 1 ; k <= na ; k++ ) {
    R_copy(d,a[k - 1]);
    for( i = 1 ; i <= L ; i++ ) {
      R_add_mul(d,c[i],a[(k - i) - 1]);
    }
    if ( R_iszero(d) ) {
      l++;
    }
    else {
      if ( R_div(A,d,dm) ) {
        ll_die("Berlekamp-Massey: dm not invertible");
      }
      R_neg(A,A);
      if ( (2 * L) >= k ) {
        ll_array_add_mul_by_cte(&c[l],nc - l,p,np,A);
        l++;
      }
      else {
        ll_vec_copy(t,c,nc);
        if ( l > nc ) {
          ll_vec_zero(&c[nc],l - nc);
          ll_vec_mul_by_cte(&c[l],p,np,A);
        }
        else {
          ll_array_add_mul_by_cte(&c[l],nc - l,p,np,A);
        }
        tmp = p;
        p = t;
        np = L + 1;
        t = tmp;
        L = k - L;
        nc = L + 1;
        R_copy(dm,d);
        l = 1;
      }
    }
  }

  nr[0] = nc;

  ll_vec_free(buf,nbuf);
  R_clear(A);
  R_clear(dm);
  R_clear(d);
}

