/*
 * This file is part of decoding
 * Copyright 2012  Guillaume Quintin, INRIA
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

/**
> `void ll_dbpol_shift_in_X(R *r,R *a,int nax,int nay,R x);`
  Set `r` to `a(X + x,Y)` where `a` has size (`nax`,`nay`).
  If your ring has caracteristic 2 (`RING_COMM_CAR2`)
  then `ll_upol_shift_car22` is called,
  otherwise `ll_upol_shift_fast2` is called.
  The dichotomy algorithm is used.
*/

void ll_dbpol_shift_in_X(R *r,R *a,int nax,int nay,R x) {
#ifdef RING_COMM_CAR2
  int ai,na;
  struct ll_upol_shift_car2_pc *pc;

  pc = ll_upol_shift_car2_prepare(nax,x);
  na = nax * nay;
  for( ai = 0 ; ai < na ; ai += nax ) {
    ll_upol_shift_car22(&r[ai],&a[ai],nax,x,pc);
  }
  ll_upol_shift_car2_finish(pc);
#else
  int ai,na;
  struct ll_upol_shift_fast_pc *pc;
  
  pc = ll_upol_shift_fast_prepare(nax,x);
  na = nax * nay;
  for( ai = 0 ; ai < na ; ai += nax ) {
    ll_upol_shift_fast2(&r[ai],&a[ai],nax,x,pc); 
  }
  ll_upol_shift_fast_finish(pc);
#endif
}

