/*
 * This file is part of decoding
 * Copyright 2012  Guillaume Quintin, INRIA
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

/* Koetter interpolation algorithm. (single multiplicity variant)
   From: The Guruswami-Sudan Decoding Algorithm for Reed-Solomon Codes.
         R. J. McEliece. */

/**
> `int ll_koetter_single(R *q,int *nqx,int *nqy,R *supp,int n,int k,\
  int L,int m,R *y,int tau);`
  Set `q` of size (`nqx`,`nqy`) to be the interpolation polynomial of
  the Guruswami-Sudan list decoding algorithm for a Reed-Solomon code
  of support `supp`, length `n`, dimension `k`, list-size `L` and for
  a received word `y` with at most `tau` errors. The multiplicity `m`
  is the same for all points (`supp[i]`,`y[i]`).
  If `q` is found `0` is returned otherwise `-1` is returned.
  The Koetter algorithm is used.
*/

int ll_koetter_single(R *q,int *nqx,int *nqy,
                      R *supp,int n,int k,
                      int L,int m,R *y,int tau)
{
  if ( m == 1 ) {
    return ll_koetter_sudan(q,nqx,nqy,supp,n,k,L,y,tau);
  }
  return ll_koetter_shift_single(q,nqx,nqy,supp,n,k,L,m,y,tau);
  /* return ll_koetter_binomial_single(q,nqx,nqy,supp,n,k,L,m,y,tau); */
}

/**
> `int ll_koetter_multi(R *q,int *nqx,int *nqy,R *supp,int n,int k,\
  int L,int *m,R *y,int tau);`
  Set `q` of size (`nqx`,`nqy`) to be the interpolation polynomial of
  the Guruswami-Sudan list decoding algorithm for a Reed-Solomon code
  of support `supp`, length `n`, dimension `k`, list-size `L` and for
  a received word `y` with at most `tau` errors. The multiplicities
  for each point (`supp[i]`,`y[i]`) is given by `m[i]`.
  If `q` is found `0` is returned otherwise `-1` is returned.
  The Koetter algorithm is used.
*/

int ll_koetter_multi(R *q,int *nqx,int *nqy,
                     R *supp,int n,int k,
                     int L,int *m,R *y,int tau)
{
  return ll_koetter_shift_multi(q,nqx,nqy,supp,n,k,L,m,y,tau);
}

