/*
 * This file is part of decoding
 * Copyright 2012  Guillaume Quintin, INRIA
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

/* NODE MANIPULATION */

/**
> `struct ll_lili_vec *ll_lili_vec_node_malloc(int na);`
  Return a `ll_lili_vec` struct containing an allocated vector of
  size `na`.
*/

struct ll_lili_vec *ll_lili_vec_node_malloc(int na) {
  struct ll_lili_vec *node;
  node = ll_malloc(sizeof(struct ll_lili_vec));
  node->next = 0;
  node->prev = 0;
  node->na = na;
  node->a = ll_vec_malloc_init(na);
  return node;
}

/**
> `struct ll_lili_vec *ll_lili_vec_node_malloc_copy(R *a,int na);`
  Return a `ll_lili_vec` struct containing a deep copy of the vector
  `a` of size `na`.
*/

struct ll_lili_vec *ll_lili_vec_node_malloc_copy(R *a,int na) {
  struct ll_lili_vec *node;
  node = ll_malloc(sizeof(struct ll_lili_vec));
  node->next = 0;
  node->prev = 0;
  node->na = na;
  node->a = ll_vec_malloc_copy(a,na);
  return node;
}

/**
> `void ll_lili_vec_node_free(struct ll_lili_vec *node);`
  Free the `ll_lili_vec` struct `node`.
*/

void ll_lili_vec_node_free(struct ll_lili_vec *node) {
  ll_vec_free(node->a,node->na);
  ll_free(node);
}

/* LINKED LIST MANIPULATION */

/**
> `void ll_lili_vec_node_attach_after(struct ll_lili_vec *after,\
  struct ll_lili_vec *node);`
  Attach the `node` after `after`.
*/

void ll_lili_vec_node_attach_after(struct ll_lili_vec *after,
                                   struct ll_lili_vec *node)
{
  struct ll_lili_vec *tmp;
  tmp = after->next;
  after->next = node;
  node->prev = after;
  node->next = tmp;
}

/**
> `void ll_lili_vec_node_attach_before(struct ll_lili_vec *before,\
  struct ll_lili_vec *node);`
  Attach `node` before `before`.
*/

void ll_lili_vec_node_attach_before(struct ll_lili_vec *before,
                                    struct ll_lili_vec *node)
{
  struct ll_lili_vec *tmp;
  tmp = before->prev;
  before->prev = node;
  node->next = before;
  node->prev = tmp;
}

/**
> `void ll_lili_vec_node_detach(struct ll_lili_vec *node);`
  Detach `node`.
*/

void ll_lili_vec_node_detach(struct ll_lili_vec *node) {
  if ( node->next ) node->next->prev = node->prev;
  if ( node->prev ) node->prev->next = node->next;
}

/**
> `void ll_lili_vec_list_free(struct ll_lili_vec *start);`
  Free the linked list whose first node is `start`.
*/

void ll_lili_vec_list_free(struct ll_lili_vec *start) {
  struct ll_lili_vec *node,*tmp;
  for( node = start ; node ; ) {
    tmp = node->next;
    ll_lili_vec_node_free(node);
    node = tmp;
  }
}

