/*
 * This file is part of decoding
 * Copyright 2012  Guillaume Quintin, INRIA
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

/**
> `void ll_upol_horner(R r,R *a,int na,R x);`
  Set `r` to be `a(x)` where `a` is a univariate polynomial
  of size `na`.
  The Horner algorithm is used.
*/

void ll_upol_horner(R r,R *a,int na,R x) {
  int i;
  R_copy(r,a[na - 1]);
  for( i = na - 2 ; i >= 0 ; i-- ) {
    R_mul(r,r,x);
    R_add(r,r,a[i]);
  }
}

/**
> `void ll_upol_multi_horner(R *r,R *a,int na,R *b,int nb);`
  Set `r` such that `r[i]` is `a(b[i])` where `a` is a
  univariate polynomial of size `na` and `b` is a vector
  of size `nb`.
*/

void ll_upol_multi_horner(R *r,R *a,int na,R *b,int nb) {
  int i;
  for( i = 0 ; i < nb ; i++ ) {
    ll_upol_horner(r[i],a,na,b[i]);
  }
}

