/*
 * This file is part of decoding
 * Copyright 2012  Guillaume Quintin, INRIA
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

void ll_upol_mul_kronecker_word_encode(mp_limb_t *r,R *a,int na,
                                       int bits)
{
#ifdef RING_MODN_WORD
  int i;
  for( i = 0 ; i < na ; i++ ) {
    int off,sh;
    off = i * bits / GMP_NUMB_BITS;
    sh = (i * bits) % GMP_NUMB_BITS;
    ll_put_word_into_mp_limb(&r[off],a[i],sh);
  }
#else
  (void)r;
  (void)a;
  (void)na;
  (void)bits;
#endif
}

void ll_upol_mul_kronecker_word_decode(R *r,int nr,mp_limb_t *sp,
                                       int bits)
{
#ifdef RING_MODN_WORD
  int i;
  if ( bits <= GMP_NUMB_BITS ) {
    mp_limb_t mask;
    ll_get_mask_1(&mask,bits);
    for( i = 0 ; i < nr ; i++ ) {
      int off,sh;
      off = i * bits / GMP_NUMB_BITS;
      sh = (i * bits) % GMP_NUMB_BITS;
      ll_get_word_from_mp_limb_1(r[i],&sp[off],sh,mask);
    }
  }
  else {
    int n;
    mp_limb_t *mask;
    n = bits / GMP_NUMB_BITS;
    if ( (bits % GMP_NUMB_BITS) > 0 ) n++;
    n++; /* n limbs can hold within at most (n + 1) limbs */
    mask = ll_malloc(n * sizeof(mp_limb_t));
    ll_get_mask_n(mask,bits);
    for( i = 0 ; i < nr ; i++ ) {
      int off,sh;
      off = i * bits / GMP_NUMB_BITS;
      sh = (i * bits) % GMP_NUMB_BITS;
      ll_get_word_from_mp_limb_n(r[i],&sp[off],n,sh,mask);
    }
    ll_free(mask);
  }
#else
  (void)r;
  (void)nr;
  (void)sp;
  (void)bits;
#endif
}

void ll_upol_mul_kronecker_word(R *r,R *a,int na,R *b,int nb) {
#ifdef RING_MODN_WORD
  int bits,ns1p,ns2p,nr;
  mp_limb_t *mem,*s1p,*s2p,*rp;
  
  bits = 2 * bitsize((unsigned long)R_characteristic) + 2 +
             bitsize((unsigned long)MIN(na,nb)) + 2;
  ns1p = na * bits / GMP_NUMB_BITS + 2;
  ns2p = nb * bits / GMP_NUMB_BITS + 2;
  mem = ll_calloc(2 * (ns1p + ns2p) * sizeof(mp_limb_t));
  s1p = mem;
  s2p = &mem[ns1p];
  rp = &mem[ns1p + ns2p];
  ll_upol_mul_kronecker_word_encode(s1p,a,na,bits);
  ll_upol_mul_kronecker_word_encode(s2p,b,nb,bits);
  if ( ns1p > ns2p ) {
    mpn_mul(rp,s1p,ns1p,s2p,ns2p);
  }
  else {
    mpn_mul(rp,s2p,ns2p,s1p,ns1p);
  }
  nr = na + nb - 1;
  ll_upol_mul_kronecker_word_decode(r,nr,rp,bits);
  ll_free(mem);
#else
  ll_upol_mul_naive(r,a,na,b,nb);
#endif
}

