/*
 * This file is part of decoding
 * Copyright 2012  Guillaume Quintin, INRIA
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

/**
> `void ll_upol_prod_affine(R *r,R *a,int na);`
  Set `r` to be the product of the `X + a[i]`
  where `a` is a vector of size `na`.
  The dichotomy algorithm is used.
*/

void ll_upol_prod_affine(R *r,R *a,int na) {
  R *tmp,*r1,*r2;
  int ntmp,nr1,nr2,na1,na2;

  if ( na <= 2 ) {
    switch(na) {
      case 0:
        R_castint(r[0],1);
        return;
      case 1:
        R_castint(r[1],1);
        R_copy(r[0],a[0]);
        return;
      case 2: {
        R_castint(r[2],1);
        R_add(r[1],a[0],a[1]);
        R_mul(r[0],a[0],a[1]);
        return;
      }
    }
  }
  na1 = na / 2;
  na2 = na - na1;
  nr1 = na1 + 1;
  nr2 = na2 + 1;
  ntmp = nr1 + nr2;
  tmp = ll_vec_malloc_init(ntmp);
  r1 = tmp;
  r2 = &tmp[nr1];
  ll_upol_prod_affine(r1,a,na1);
  ll_upol_prod_affine(r2,&a[na1],na2);
  ll_upol_mul(r,r1,nr1,r2,nr2);
  ll_vec_free(tmp,ntmp);
}

/**
> `void ll_upol_prod_affine_neg(R *r,R *a,int na);`
  Set `r` to be the product of the `X - a[i]`
  where `a` is a vector of size `na`.
  The dichotomy algorithm is used.
*/

void ll_upol_prod_affine_neg(R *r,R *a,int na) {
  R *tmp,*r1,*r2;
  int ntmp,nr1,nr2,na1,na2;

  if ( na <= 2 ) {
    switch(na) {
      case 0:
        R_castint(r[0],1);
        return;
      case 1:
        R_castint(r[1],1);
        R_neg(r[0],a[0]);
        return;
      case 2: {
        R_castint(r[2],1);
        R_zero(r[1]);
        R_sub_sub(r[1],a[0],a[1]);
        R_mul(r[0],a[0],a[1]);
        return;
      }
    }
  }
  na1 = na / 2;
  na2 = na - na1;
  nr1 = na1 + 1;
  nr2 = na2 + 1;
  ntmp = nr1 + nr2;
  tmp = ll_vec_malloc_init(ntmp);
  r1 = tmp;
  r2 = &tmp[nr1];
  ll_upol_prod_affine_neg(r1,a,na1);
  ll_upol_prod_affine_neg(r2,&a[na1],na2);
  ll_upol_mul(r,r1,nr1,r2,nr2);
  ll_vec_free(tmp,ntmp);
}

