/*
 * This file is part of decoding
 * Copyright 2012  Guillaume Quintin, INRIA
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

/**
> `void ll_die(const char *msg);`
  Print `msg` to `stderr` and terminate the program
  with exit code `-1`.
*/

void ll_die(const char *msg) {
#ifndef R
  fprintf(stderr,"%s\n",msg);
#else
  fprintf(stderr,__S(R) ": %s\n",msg);
#endif
  exit(-1);
}

/* internal functions */

void ll_put_word_into_mp_limb(mp_limb_t *r,R a,int sh) {
#ifdef RING_MODN_WORD
  mp_limb_t tmp;
  tmp = (mp_limb_t)(a[0]);
  r[0] |= (tmp << sh) & GMP_NUMB_MASK;
  if ( sh > 0 ) {
    r[1] |= (tmp >> (sizeof(mp_limb_t) * 8 - sh)) & GMP_NUMB_MASK;
  }
#else
  (void)r;
  (void)a;
  (void)sh;
#endif
}

void ll_get_word_from_mp_limb_1(R r,mp_limb_t *a,int sh,
                                mp_limb_t mask)
{
#ifdef RING_MODN_WORD
  mp_limb_t tmp;
  tmp = a[0] >> sh;
  if ( sh > 0 ) {
    tmp |= a[1] << (sizeof(mp_limb_t) * 8 - sh);
  }
  tmp &= mask;
  r[0] = (R_content)(tmp % (mp_limb_t)R_characteristic);
#else
  (void)r;
  (void)a;
  (void)sh;
  (void)mask;
#endif
}

void ll_get_word_from_mp_limb_n(R r,mp_limb_t *a,int n,int sh,
                                mp_limb_t *mask)
{
#ifdef RING_MODN_WORD
#define HELPER ((sizeof(int) * 8 + (2 * GMP_NUMB_BITS) + 4) \
                / GMP_NUMB_BITS + 1)
  int i;
  mp_limb_t rem,tmp0[HELPER],tmp1[HELPER];
  for( i = 0 ; i < n ; i++ ) {
    tmp0[i] = a[i];
  }
  if ( sh == 0) {
    ll_mpn_and_n(tmp0,mask,n);
    rem = mpn_mod_1(tmp0,n,(mp_limb_t)R_characteristic);
    r[0] = (R_content)rem;
  }
  else {
    mpn_rshift(tmp1,tmp0,n,sh);
    ll_mpn_and_n(tmp1,mask,n);
    rem = mpn_mod_1(tmp1,n,(mp_limb_t)R_characteristic);
    r[0] = (R_content)rem;
  }
#undef HELPER
#else
  (void)r;
  (void)a;
  (void)n;
  (void)sh;
  (void)mask;
#endif
}

