/*
 * This file is part of decoding
 * Copyright 2012  Guillaume Quintin, INRIA
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

/**
> `mpfq_gf2n_wrapper.c`
  Implement all fields `GF(2^n)` with `2 <= s <= 255`. It is a
  wrapper to MPFQ. The `DEGREE` indicates which extension of
  `GF(2)` you want to use.
  
  ``
  #define DEGREE 8
  #include <decoding/rings/mpfq_gf2n_wrapper.c>
  #include <decoding/algos.c>
  ``
  
  In the example above, the name of the finite field will be
  `gf2_8`. You can of course use several MPFQ fields in the
  same program.

  ``
  #define DEGREE 8
  #include <decoding/rings/mpfq_gf2n_wrapper.c>
  #include <decoding/algos.c>
  
  #define DEGREE 64
  #include <decoding/rings/mpfq_gf2n_wrapper.c>
  #include <decoding/algos.c>
  ``

  It will then generate the code for the two finite fields
  GF(2^8) and GF(2^64). You will then have two types.

  - `gf2_8` corresponding to GF(7).

  - `gf2_64` corresponding to GF(11).

  The degree of the extension is accessible with the
  `R_ext_degree` variable. For exemple we will have the
  following.

  - `gf2_8_ext_degree = 8`.

  - `gf2_64_ext_degree = 64`.
*/

#if DEGREE==2
#include <mpfq/mpfq_2_2.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_2_wrapper.c>
#endif

#if DEGREE==3
#include <mpfq/mpfq_2_3.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_3_wrapper.c>
#endif

#if DEGREE==4
#include <mpfq/mpfq_2_4.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_4_wrapper.c>
#endif

#if DEGREE==5
#include <mpfq/mpfq_2_5.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_5_wrapper.c>
#endif

#if DEGREE==6
#include <mpfq/mpfq_2_6.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_6_wrapper.c>
#endif

#if DEGREE==7
#include <mpfq/mpfq_2_7.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_7_wrapper.c>
#endif

#if DEGREE==8
#include <mpfq/mpfq_2_8.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_8_wrapper.c>
#endif

#if DEGREE==9
#include <mpfq/mpfq_2_9.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_9_wrapper.c>
#endif

#if DEGREE==10
#include <mpfq/mpfq_2_10.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_10_wrapper.c>
#endif

#if DEGREE==11
#include <mpfq/mpfq_2_11.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_11_wrapper.c>
#endif

#if DEGREE==12
#include <mpfq/mpfq_2_12.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_12_wrapper.c>
#endif

#if DEGREE==13
#include <mpfq/mpfq_2_13.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_13_wrapper.c>
#endif

#if DEGREE==14
#include <mpfq/mpfq_2_14.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_14_wrapper.c>
#endif

#if DEGREE==15
#include <mpfq/mpfq_2_15.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_15_wrapper.c>
#endif

#if DEGREE==16
#include <mpfq/mpfq_2_16.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_16_wrapper.c>
#endif

#if DEGREE==17
#include <mpfq/mpfq_2_17.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_17_wrapper.c>
#endif

#if DEGREE==18
#include <mpfq/mpfq_2_18.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_18_wrapper.c>
#endif

#if DEGREE==19
#include <mpfq/mpfq_2_19.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_19_wrapper.c>
#endif

#if DEGREE==20
#include <mpfq/mpfq_2_20.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_20_wrapper.c>
#endif

#if DEGREE==21
#include <mpfq/mpfq_2_21.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_21_wrapper.c>
#endif

#if DEGREE==22
#include <mpfq/mpfq_2_22.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_22_wrapper.c>
#endif

#if DEGREE==23
#include <mpfq/mpfq_2_23.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_23_wrapper.c>
#endif

#if DEGREE==24
#include <mpfq/mpfq_2_24.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_24_wrapper.c>
#endif

#if DEGREE==25
#include <mpfq/mpfq_2_25.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_25_wrapper.c>
#endif

#if DEGREE==26
#include <mpfq/mpfq_2_26.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_26_wrapper.c>
#endif

#if DEGREE==27
#include <mpfq/mpfq_2_27.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_27_wrapper.c>
#endif

#if DEGREE==28
#include <mpfq/mpfq_2_28.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_28_wrapper.c>
#endif

#if DEGREE==29
#include <mpfq/mpfq_2_29.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_29_wrapper.c>
#endif

#if DEGREE==30
#include <mpfq/mpfq_2_30.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_30_wrapper.c>
#endif

#if DEGREE==31
#include <mpfq/mpfq_2_31.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_31_wrapper.c>
#endif

#if DEGREE==32
#include <mpfq/mpfq_2_32.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_32_wrapper.c>
#endif

#if DEGREE==33
#include <mpfq/mpfq_2_33.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_33_wrapper.c>
#endif

#if DEGREE==34
#include <mpfq/mpfq_2_34.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_34_wrapper.c>
#endif

#if DEGREE==35
#include <mpfq/mpfq_2_35.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_35_wrapper.c>
#endif

#if DEGREE==36
#include <mpfq/mpfq_2_36.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_36_wrapper.c>
#endif

#if DEGREE==37
#include <mpfq/mpfq_2_37.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_37_wrapper.c>
#endif

#if DEGREE==38
#include <mpfq/mpfq_2_38.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_38_wrapper.c>
#endif

#if DEGREE==39
#include <mpfq/mpfq_2_39.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_39_wrapper.c>
#endif

#if DEGREE==40
#include <mpfq/mpfq_2_40.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_40_wrapper.c>
#endif

#if DEGREE==41
#include <mpfq/mpfq_2_41.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_41_wrapper.c>
#endif

#if DEGREE==42
#include <mpfq/mpfq_2_42.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_42_wrapper.c>
#endif

#if DEGREE==43
#include <mpfq/mpfq_2_43.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_43_wrapper.c>
#endif

#if DEGREE==44
#include <mpfq/mpfq_2_44.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_44_wrapper.c>
#endif

#if DEGREE==45
#include <mpfq/mpfq_2_45.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_45_wrapper.c>
#endif

#if DEGREE==46
#include <mpfq/mpfq_2_46.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_46_wrapper.c>
#endif

#if DEGREE==47
#include <mpfq/mpfq_2_47.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_47_wrapper.c>
#endif

#if DEGREE==48
#include <mpfq/mpfq_2_48.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_48_wrapper.c>
#endif

#if DEGREE==49
#include <mpfq/mpfq_2_49.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_49_wrapper.c>
#endif

#if DEGREE==50
#include <mpfq/mpfq_2_50.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_50_wrapper.c>
#endif

#if DEGREE==51
#include <mpfq/mpfq_2_51.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_51_wrapper.c>
#endif

#if DEGREE==52
#include <mpfq/mpfq_2_52.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_52_wrapper.c>
#endif

#if DEGREE==53
#include <mpfq/mpfq_2_53.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_53_wrapper.c>
#endif

#if DEGREE==54
#include <mpfq/mpfq_2_54.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_54_wrapper.c>
#endif

#if DEGREE==55
#include <mpfq/mpfq_2_55.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_55_wrapper.c>
#endif

#if DEGREE==56
#include <mpfq/mpfq_2_56.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_56_wrapper.c>
#endif

#if DEGREE==57
#include <mpfq/mpfq_2_57.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_57_wrapper.c>
#endif

#if DEGREE==58
#include <mpfq/mpfq_2_58.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_58_wrapper.c>
#endif

#if DEGREE==59
#include <mpfq/mpfq_2_59.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_59_wrapper.c>
#endif

#if DEGREE==60
#include <mpfq/mpfq_2_60.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_60_wrapper.c>
#endif

#if DEGREE==61
#include <mpfq/mpfq_2_61.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_61_wrapper.c>
#endif

#if DEGREE==62
#include <mpfq/mpfq_2_62.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_62_wrapper.c>
#endif

#if DEGREE==63
#include <mpfq/mpfq_2_63.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_63_wrapper.c>
#endif

#if DEGREE==64
#include <mpfq/mpfq_2_64.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_64_wrapper.c>
#endif

#if DEGREE==65
#include <mpfq/mpfq_2_65.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_65_wrapper.c>
#endif

#if DEGREE==66
#include <mpfq/mpfq_2_66.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_66_wrapper.c>
#endif

#if DEGREE==67
#include <mpfq/mpfq_2_67.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_67_wrapper.c>
#endif

#if DEGREE==68
#include <mpfq/mpfq_2_68.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_68_wrapper.c>
#endif

#if DEGREE==69
#include <mpfq/mpfq_2_69.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_69_wrapper.c>
#endif

#if DEGREE==70
#include <mpfq/mpfq_2_70.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_70_wrapper.c>
#endif

#if DEGREE==71
#include <mpfq/mpfq_2_71.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_71_wrapper.c>
#endif

#if DEGREE==72
#include <mpfq/mpfq_2_72.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_72_wrapper.c>
#endif

#if DEGREE==73
#include <mpfq/mpfq_2_73.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_73_wrapper.c>
#endif

#if DEGREE==74
#include <mpfq/mpfq_2_74.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_74_wrapper.c>
#endif

#if DEGREE==75
#include <mpfq/mpfq_2_75.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_75_wrapper.c>
#endif

#if DEGREE==76
#include <mpfq/mpfq_2_76.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_76_wrapper.c>
#endif

#if DEGREE==77
#include <mpfq/mpfq_2_77.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_77_wrapper.c>
#endif

#if DEGREE==78
#include <mpfq/mpfq_2_78.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_78_wrapper.c>
#endif

#if DEGREE==79
#include <mpfq/mpfq_2_79.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_79_wrapper.c>
#endif

#if DEGREE==80
#include <mpfq/mpfq_2_80.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_80_wrapper.c>
#endif

#if DEGREE==81
#include <mpfq/mpfq_2_81.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_81_wrapper.c>
#endif

#if DEGREE==82
#include <mpfq/mpfq_2_82.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_82_wrapper.c>
#endif

#if DEGREE==83
#include <mpfq/mpfq_2_83.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_83_wrapper.c>
#endif

#if DEGREE==84
#include <mpfq/mpfq_2_84.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_84_wrapper.c>
#endif

#if DEGREE==85
#include <mpfq/mpfq_2_85.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_85_wrapper.c>
#endif

#if DEGREE==86
#include <mpfq/mpfq_2_86.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_86_wrapper.c>
#endif

#if DEGREE==87
#include <mpfq/mpfq_2_87.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_87_wrapper.c>
#endif

#if DEGREE==88
#include <mpfq/mpfq_2_88.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_88_wrapper.c>
#endif

#if DEGREE==89
#include <mpfq/mpfq_2_89.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_89_wrapper.c>
#endif

#if DEGREE==90
#include <mpfq/mpfq_2_90.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_90_wrapper.c>
#endif

#if DEGREE==91
#include <mpfq/mpfq_2_91.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_91_wrapper.c>
#endif

#if DEGREE==92
#include <mpfq/mpfq_2_92.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_92_wrapper.c>
#endif

#if DEGREE==93
#include <mpfq/mpfq_2_93.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_93_wrapper.c>
#endif

#if DEGREE==94
#include <mpfq/mpfq_2_94.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_94_wrapper.c>
#endif

#if DEGREE==95
#include <mpfq/mpfq_2_95.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_95_wrapper.c>
#endif

#if DEGREE==96
#include <mpfq/mpfq_2_96.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_96_wrapper.c>
#endif

#if DEGREE==97
#include <mpfq/mpfq_2_97.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_97_wrapper.c>
#endif

#if DEGREE==98
#include <mpfq/mpfq_2_98.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_98_wrapper.c>
#endif

#if DEGREE==99
#include <mpfq/mpfq_2_99.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_99_wrapper.c>
#endif

#if DEGREE==100
#include <mpfq/mpfq_2_100.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_100_wrapper.c>
#endif

#if DEGREE==101
#include <mpfq/mpfq_2_101.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_101_wrapper.c>
#endif

#if DEGREE==102
#include <mpfq/mpfq_2_102.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_102_wrapper.c>
#endif

#if DEGREE==103
#include <mpfq/mpfq_2_103.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_103_wrapper.c>
#endif

#if DEGREE==104
#include <mpfq/mpfq_2_104.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_104_wrapper.c>
#endif

#if DEGREE==105
#include <mpfq/mpfq_2_105.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_105_wrapper.c>
#endif

#if DEGREE==106
#include <mpfq/mpfq_2_106.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_106_wrapper.c>
#endif

#if DEGREE==107
#include <mpfq/mpfq_2_107.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_107_wrapper.c>
#endif

#if DEGREE==108
#include <mpfq/mpfq_2_108.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_108_wrapper.c>
#endif

#if DEGREE==109
#include <mpfq/mpfq_2_109.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_109_wrapper.c>
#endif

#if DEGREE==110
#include <mpfq/mpfq_2_110.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_110_wrapper.c>
#endif

#if DEGREE==111
#include <mpfq/mpfq_2_111.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_111_wrapper.c>
#endif

#if DEGREE==112
#include <mpfq/mpfq_2_112.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_112_wrapper.c>
#endif

#if DEGREE==113
#include <mpfq/mpfq_2_113.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_113_wrapper.c>
#endif

#if DEGREE==114
#include <mpfq/mpfq_2_114.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_114_wrapper.c>
#endif

#if DEGREE==115
#include <mpfq/mpfq_2_115.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_115_wrapper.c>
#endif

#if DEGREE==116
#include <mpfq/mpfq_2_116.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_116_wrapper.c>
#endif

#if DEGREE==117
#include <mpfq/mpfq_2_117.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_117_wrapper.c>
#endif

#if DEGREE==118
#include <mpfq/mpfq_2_118.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_118_wrapper.c>
#endif

#if DEGREE==119
#include <mpfq/mpfq_2_119.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_119_wrapper.c>
#endif

#if DEGREE==120
#include <mpfq/mpfq_2_120.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_120_wrapper.c>
#endif

#if DEGREE==121
#include <mpfq/mpfq_2_121.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_121_wrapper.c>
#endif

#if DEGREE==122
#include <mpfq/mpfq_2_122.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_122_wrapper.c>
#endif

#if DEGREE==123
#include <mpfq/mpfq_2_123.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_123_wrapper.c>
#endif

#if DEGREE==124
#include <mpfq/mpfq_2_124.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_124_wrapper.c>
#endif

#if DEGREE==125
#include <mpfq/mpfq_2_125.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_125_wrapper.c>
#endif

#if DEGREE==126
#include <mpfq/mpfq_2_126.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_126_wrapper.c>
#endif

#if DEGREE==127
#include <mpfq/mpfq_2_127.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_127_wrapper.c>
#endif

#if DEGREE==128
#include <mpfq/mpfq_2_128.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_128_wrapper.c>
#endif

#if DEGREE==129
#include <mpfq/mpfq_2_129.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_129_wrapper.c>
#endif

#if DEGREE==130
#include <mpfq/mpfq_2_130.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_130_wrapper.c>
#endif

#if DEGREE==131
#include <mpfq/mpfq_2_131.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_131_wrapper.c>
#endif

#if DEGREE==132
#include <mpfq/mpfq_2_132.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_132_wrapper.c>
#endif

#if DEGREE==133
#include <mpfq/mpfq_2_133.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_133_wrapper.c>
#endif

#if DEGREE==134
#include <mpfq/mpfq_2_134.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_134_wrapper.c>
#endif

#if DEGREE==135
#include <mpfq/mpfq_2_135.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_135_wrapper.c>
#endif

#if DEGREE==136
#include <mpfq/mpfq_2_136.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_136_wrapper.c>
#endif

#if DEGREE==137
#include <mpfq/mpfq_2_137.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_137_wrapper.c>
#endif

#if DEGREE==138
#include <mpfq/mpfq_2_138.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_138_wrapper.c>
#endif

#if DEGREE==139
#include <mpfq/mpfq_2_139.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_139_wrapper.c>
#endif

#if DEGREE==140
#include <mpfq/mpfq_2_140.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_140_wrapper.c>
#endif

#if DEGREE==141
#include <mpfq/mpfq_2_141.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_141_wrapper.c>
#endif

#if DEGREE==142
#include <mpfq/mpfq_2_142.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_142_wrapper.c>
#endif

#if DEGREE==143
#include <mpfq/mpfq_2_143.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_143_wrapper.c>
#endif

#if DEGREE==144
#include <mpfq/mpfq_2_144.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_144_wrapper.c>
#endif

#if DEGREE==145
#include <mpfq/mpfq_2_145.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_145_wrapper.c>
#endif

#if DEGREE==146
#include <mpfq/mpfq_2_146.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_146_wrapper.c>
#endif

#if DEGREE==147
#include <mpfq/mpfq_2_147.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_147_wrapper.c>
#endif

#if DEGREE==148
#include <mpfq/mpfq_2_148.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_148_wrapper.c>
#endif

#if DEGREE==149
#include <mpfq/mpfq_2_149.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_149_wrapper.c>
#endif

#if DEGREE==150
#include <mpfq/mpfq_2_150.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_150_wrapper.c>
#endif

#if DEGREE==151
#include <mpfq/mpfq_2_151.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_151_wrapper.c>
#endif

#if DEGREE==152
#include <mpfq/mpfq_2_152.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_152_wrapper.c>
#endif

#if DEGREE==153
#include <mpfq/mpfq_2_153.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_153_wrapper.c>
#endif

#if DEGREE==154
#include <mpfq/mpfq_2_154.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_154_wrapper.c>
#endif

#if DEGREE==155
#include <mpfq/mpfq_2_155.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_155_wrapper.c>
#endif

#if DEGREE==156
#include <mpfq/mpfq_2_156.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_156_wrapper.c>
#endif

#if DEGREE==157
#include <mpfq/mpfq_2_157.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_157_wrapper.c>
#endif

#if DEGREE==158
#include <mpfq/mpfq_2_158.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_158_wrapper.c>
#endif

#if DEGREE==159
#include <mpfq/mpfq_2_159.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_159_wrapper.c>
#endif

#if DEGREE==160
#include <mpfq/mpfq_2_160.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_160_wrapper.c>
#endif

#if DEGREE==161
#include <mpfq/mpfq_2_161.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_161_wrapper.c>
#endif

#if DEGREE==162
#include <mpfq/mpfq_2_162.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_162_wrapper.c>
#endif

#if DEGREE==163
#include <mpfq/mpfq_2_163.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_163_wrapper.c>
#endif

#if DEGREE==164
#include <mpfq/mpfq_2_164.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_164_wrapper.c>
#endif

#if DEGREE==165
#include <mpfq/mpfq_2_165.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_165_wrapper.c>
#endif

#if DEGREE==166
#include <mpfq/mpfq_2_166.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_166_wrapper.c>
#endif

#if DEGREE==167
#include <mpfq/mpfq_2_167.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_167_wrapper.c>
#endif

#if DEGREE==168
#include <mpfq/mpfq_2_168.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_168_wrapper.c>
#endif

#if DEGREE==169
#include <mpfq/mpfq_2_169.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_169_wrapper.c>
#endif

#if DEGREE==170
#include <mpfq/mpfq_2_170.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_170_wrapper.c>
#endif

#if DEGREE==171
#include <mpfq/mpfq_2_171.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_171_wrapper.c>
#endif

#if DEGREE==172
#include <mpfq/mpfq_2_172.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_172_wrapper.c>
#endif

#if DEGREE==173
#include <mpfq/mpfq_2_173.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_173_wrapper.c>
#endif

#if DEGREE==174
#include <mpfq/mpfq_2_174.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_174_wrapper.c>
#endif

#if DEGREE==175
#include <mpfq/mpfq_2_175.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_175_wrapper.c>
#endif

#if DEGREE==176
#include <mpfq/mpfq_2_176.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_176_wrapper.c>
#endif

#if DEGREE==177
#include <mpfq/mpfq_2_177.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_177_wrapper.c>
#endif

#if DEGREE==178
#include <mpfq/mpfq_2_178.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_178_wrapper.c>
#endif

#if DEGREE==179
#include <mpfq/mpfq_2_179.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_179_wrapper.c>
#endif

#if DEGREE==180
#include <mpfq/mpfq_2_180.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_180_wrapper.c>
#endif

#if DEGREE==181
#include <mpfq/mpfq_2_181.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_181_wrapper.c>
#endif

#if DEGREE==182
#include <mpfq/mpfq_2_182.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_182_wrapper.c>
#endif

#if DEGREE==183
#include <mpfq/mpfq_2_183.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_183_wrapper.c>
#endif

#if DEGREE==184
#include <mpfq/mpfq_2_184.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_184_wrapper.c>
#endif

#if DEGREE==185
#include <mpfq/mpfq_2_185.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_185_wrapper.c>
#endif

#if DEGREE==186
#include <mpfq/mpfq_2_186.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_186_wrapper.c>
#endif

#if DEGREE==187
#include <mpfq/mpfq_2_187.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_187_wrapper.c>
#endif

#if DEGREE==188
#include <mpfq/mpfq_2_188.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_188_wrapper.c>
#endif

#if DEGREE==189
#include <mpfq/mpfq_2_189.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_189_wrapper.c>
#endif

#if DEGREE==190
#include <mpfq/mpfq_2_190.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_190_wrapper.c>
#endif

#if DEGREE==191
#include <mpfq/mpfq_2_191.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_191_wrapper.c>
#endif

#if DEGREE==192
#include <mpfq/mpfq_2_192.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_192_wrapper.c>
#endif

#if DEGREE==193
#include <mpfq/mpfq_2_193.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_193_wrapper.c>
#endif

#if DEGREE==194
#include <mpfq/mpfq_2_194.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_194_wrapper.c>
#endif

#if DEGREE==195
#include <mpfq/mpfq_2_195.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_195_wrapper.c>
#endif

#if DEGREE==196
#include <mpfq/mpfq_2_196.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_196_wrapper.c>
#endif

#if DEGREE==197
#include <mpfq/mpfq_2_197.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_197_wrapper.c>
#endif

#if DEGREE==198
#include <mpfq/mpfq_2_198.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_198_wrapper.c>
#endif

#if DEGREE==199
#include <mpfq/mpfq_2_199.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_199_wrapper.c>
#endif

#if DEGREE==200
#include <mpfq/mpfq_2_200.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_200_wrapper.c>
#endif

#if DEGREE==201
#include <mpfq/mpfq_2_201.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_201_wrapper.c>
#endif

#if DEGREE==202
#include <mpfq/mpfq_2_202.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_202_wrapper.c>
#endif

#if DEGREE==203
#include <mpfq/mpfq_2_203.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_203_wrapper.c>
#endif

#if DEGREE==204
#include <mpfq/mpfq_2_204.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_204_wrapper.c>
#endif

#if DEGREE==205
#include <mpfq/mpfq_2_205.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_205_wrapper.c>
#endif

#if DEGREE==206
#include <mpfq/mpfq_2_206.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_206_wrapper.c>
#endif

#if DEGREE==207
#include <mpfq/mpfq_2_207.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_207_wrapper.c>
#endif

#if DEGREE==208
#include <mpfq/mpfq_2_208.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_208_wrapper.c>
#endif

#if DEGREE==209
#include <mpfq/mpfq_2_209.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_209_wrapper.c>
#endif

#if DEGREE==210
#include <mpfq/mpfq_2_210.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_210_wrapper.c>
#endif

#if DEGREE==211
#include <mpfq/mpfq_2_211.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_211_wrapper.c>
#endif

#if DEGREE==212
#include <mpfq/mpfq_2_212.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_212_wrapper.c>
#endif

#if DEGREE==213
#include <mpfq/mpfq_2_213.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_213_wrapper.c>
#endif

#if DEGREE==214
#include <mpfq/mpfq_2_214.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_214_wrapper.c>
#endif

#if DEGREE==215
#include <mpfq/mpfq_2_215.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_215_wrapper.c>
#endif

#if DEGREE==216
#include <mpfq/mpfq_2_216.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_216_wrapper.c>
#endif

#if DEGREE==217
#include <mpfq/mpfq_2_217.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_217_wrapper.c>
#endif

#if DEGREE==218
#include <mpfq/mpfq_2_218.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_218_wrapper.c>
#endif

#if DEGREE==219
#include <mpfq/mpfq_2_219.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_219_wrapper.c>
#endif

#if DEGREE==220
#include <mpfq/mpfq_2_220.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_220_wrapper.c>
#endif

#if DEGREE==221
#include <mpfq/mpfq_2_221.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_221_wrapper.c>
#endif

#if DEGREE==222
#include <mpfq/mpfq_2_222.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_222_wrapper.c>
#endif

#if DEGREE==223
#include <mpfq/mpfq_2_223.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_223_wrapper.c>
#endif

#if DEGREE==224
#include <mpfq/mpfq_2_224.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_224_wrapper.c>
#endif

#if DEGREE==225
#include <mpfq/mpfq_2_225.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_225_wrapper.c>
#endif

#if DEGREE==226
#include <mpfq/mpfq_2_226.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_226_wrapper.c>
#endif

#if DEGREE==227
#include <mpfq/mpfq_2_227.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_227_wrapper.c>
#endif

#if DEGREE==228
#include <mpfq/mpfq_2_228.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_228_wrapper.c>
#endif

#if DEGREE==229
#include <mpfq/mpfq_2_229.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_229_wrapper.c>
#endif

#if DEGREE==230
#include <mpfq/mpfq_2_230.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_230_wrapper.c>
#endif

#if DEGREE==231
#include <mpfq/mpfq_2_231.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_231_wrapper.c>
#endif

#if DEGREE==232
#include <mpfq/mpfq_2_232.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_232_wrapper.c>
#endif

#if DEGREE==233
#include <mpfq/mpfq_2_233.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_233_wrapper.c>
#endif

#if DEGREE==234
#include <mpfq/mpfq_2_234.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_234_wrapper.c>
#endif

#if DEGREE==235
#include <mpfq/mpfq_2_235.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_235_wrapper.c>
#endif

#if DEGREE==236
#include <mpfq/mpfq_2_236.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_236_wrapper.c>
#endif

#if DEGREE==237
#include <mpfq/mpfq_2_237.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_237_wrapper.c>
#endif

#if DEGREE==238
#include <mpfq/mpfq_2_238.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_238_wrapper.c>
#endif

#if DEGREE==239
#include <mpfq/mpfq_2_239.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_239_wrapper.c>
#endif

#if DEGREE==240
#include <mpfq/mpfq_2_240.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_240_wrapper.c>
#endif

#if DEGREE==241
#include <mpfq/mpfq_2_241.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_241_wrapper.c>
#endif

#if DEGREE==242
#include <mpfq/mpfq_2_242.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_242_wrapper.c>
#endif

#if DEGREE==243
#include <mpfq/mpfq_2_243.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_243_wrapper.c>
#endif

#if DEGREE==244
#include <mpfq/mpfq_2_244.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_244_wrapper.c>
#endif

#if DEGREE==245
#include <mpfq/mpfq_2_245.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_245_wrapper.c>
#endif

#if DEGREE==246
#include <mpfq/mpfq_2_246.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_246_wrapper.c>
#endif

#if DEGREE==247
#include <mpfq/mpfq_2_247.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_247_wrapper.c>
#endif

#if DEGREE==248
#include <mpfq/mpfq_2_248.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_248_wrapper.c>
#endif

#if DEGREE==249
#include <mpfq/mpfq_2_249.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_249_wrapper.c>
#endif

#if DEGREE==250
#include <mpfq/mpfq_2_250.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_250_wrapper.c>
#endif

#if DEGREE==251
#include <mpfq/mpfq_2_251.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_251_wrapper.c>
#endif

#if DEGREE==252
#include <mpfq/mpfq_2_252.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_252_wrapper.c>
#endif

#if DEGREE==253
#include <mpfq/mpfq_2_253.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_253_wrapper.c>
#endif

#if DEGREE==254
#include <mpfq/mpfq_2_254.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_254_wrapper.c>
#endif

#if DEGREE==255
#include <mpfq/mpfq_2_255.h>
#include <decoding/rings/mpfq_wrappers/mpfq_gf2n_2_255_wrapper.c>
#endif

