/*
 * This file is part of decoding
 * Copyright 2012  Guillaume Quintin, INRIA
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

/**
> `mpfq_gf2n_wrapper.c`
  Implement all fields `GF(2^n)` with `2 <= s <= 255`. It is a
  wrapper to MPFQ. The `DEGREE` indicates which extension of
  `GF(2)` you want to use.
  
  ``
  #define DEGREE 8
  #include <decoding/rings/mpfq_gf2n_wrapper.c>
  #include <decoding/algos.c>
  ``
  
  In the example above, the name of the finite field will be
  `gf2_8`. You can of course use several MPFQ fields in the
  same program.

  ``
  #define DEGREE 8
  #include <decoding/rings/mpfq_gf2n_wrapper.c>
  #include <decoding/algos.c>
  
  #define DEGREE 64
  #include <decoding/rings/mpfq_gf2n_wrapper.c>
  #include <decoding/algos.c>
  ``

  It will then generate the code for the two finite fields
  GF(2^8) and GF(2^64). You will then have two types.

  - `gf2_8` corresponding to GF(7).

  - `gf2_64` corresponding to GF(11).

  The degree of the extension is accessible with the
  `R_ext_degree` variable. For exemple we will have the
  following.

  - `gf2_8_ext_degree = 8`.

  - `gf2_64_ext_degree = 64`.
*/

