/*
 * This file is part of decoding
 * Copyright 2012  Guillaume Quintin, INRIA
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

/* structs */
struct sugar_vec {
  R *a;
  int na; /* math size */
  int sz; /* real size in memory */
};
typedef struct sugar_vec *vec_ptr;
typedef struct sugar_vec vec[1];

struct sugar_rs_code {
  int n,k;     /* length and dimension */
  vec supp;    /* the support */
  /* a temp zone */
  R *tmp;
  int ntmp;
  /* Sudan parameters */
  int sL,snx,stau;
  /* Guruswami-Sudan parameters */
  int gsL,gsnx,gstau,gsm;
};
typedef struct sugar_rs_code *rs_code_ptr;
typedef struct sugar_rs_code rs_code[1];

/* sugar functions */

/* sugar/vec.c */
void vec_init(vec_ptr);
void vec_inits(vec_ptr,...);
void vec_clear(vec_ptr);
void vec_clears(vec_ptr,...);
void vec_adjust_size(vec,int);
void vec_zero(vec);
void vec_append(vec,R);
void vec_random(vec);
void vec_random_hamming_weight(vec,int,int);
void vec_add(vec,vec,vec);
void vec_sub(vec,vec,vec);
void vec_mul_by_cte(vec,vec,R);
void vec_set_coeff(vec,R,int);
void vec_get_coeff(R,vec,int);
int vec_get_len(vec);
int vec_hamming_weight(vec);
int vec_hamming_distance(vec,vec);
void vec_copy(vec,vec);
int vec_cmp(vec,vec);
void vec_ptr_clear(vec*,int);
void vec_print(FILE*,vec_ptr);

/* sugar/rs.c */
void rs_code_init(rs_code,int,int);
void rs_code_init_with_support(rs_code,int,int);
void rs_code_clear(rs_code);
void rs_code_random_codeword(vec,rs_code);
int rs_code_sudan_koetter(vec**,rs_code,vec,int);
int rs_code_guruswami_sudan_koetter(vec**,rs_code,vec,int);

/* sugar/print.c */
void decoding_printf(FILE*,const char*,...);

