/*
 * This file is part of decoding
 * Copyright 2012  Guillaume Quintin, INRIA
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */



#include <stdlib.h>
#include <decoding/decoding.h> 



/* This exemple shows how to use the
   Guruswami-sudan algorithm. */



/* Note that you can give costum names to
   rings and fields with RING_NAME. */

#define RING_NAME ring

/* Generate code for GF(PRIME) where PRIME
   holds within a machine word. */

#define PRIME 101
#include <decoding/rings/gfp_word.c>
#include <decoding/algos.c>



int main(void) {

  int nc,i;

  /* We need three vectors: the message -------> m 
                            the error ---------> e
                            the received word -> y */
  ring_vec m,e,y;

  
  /* We need an array of vectors. It will receive
     the list of vectors returned by the Guruswami-Sudan
     algorithm. */
  ring_vec *c;
  
  
  /* We need a Reed-Solomon code over ring. */
  ring_rs_code rs;

  /* We initialize the ring. */
  ring_ring_init();


  /* We initialize the vectors. */
  ring_vec_inits(m,e,y,0);

  
  /* We make a Reed-Solomon code of parameters [50,10,41] over 
     our ring. Note that all the functions are prefixed by
     ``ring_". */
  ring_rs_code_init_with_support(rs,50,10);

  
  /* We generate a random codeword. This is our message m. */
  ring_rs_code_random_codeword(m,rs);

  
  /* We generate an error vector e of length 50 and weight 22. */
  ring_vec_random_hamming_weight(e,50,22);

  
  /* We build the received word y = m + e. */
  ring_vec_add(y,m,e);

  
  /* We call the Sudan algorithm to correct y. We tell the algorithm
     that we want to correct the maximum possible number of errors.
     The list of codewords returned by our algorithm will be put
     in c. */
  nc = ring_rs_code_sudan_koetter(&c,rs,y,SUDAN_MAX_ERRORS);

  
  /* nc contains the number of codewords in the array c. Then
     we print the original message m and the list returned by
     the algorithm. */
  ring_decoding_printf(stdout,"Original message: %v\n",m);
  for( i = 0 ; i < nc ; i++ ) {
    ring_decoding_printf(stdout,"Codeword %d: %v\n",i,c[i]);
  }

  return 0;

}

