/*
 * This file is part of decoding
 * Copyright 2012  Guillaume Quintin, Morgan Barbier, INRIA
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#define _POSIX_C_SOURCE 1

#include <time.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <decoding/decoding.h>

#define RING_NAME fld
#define PRIME 5
#include <decoding/rings/gfp_word.c>
#include <decoding/algos.c>

void mtest_berlekamp_massey(FILE *out,int n1,int n2) {
  int i,nlambda;
  fld *s,*lambda;
  s = fld_ll_vec_malloc_init(n2 + 1);
  lambda = fld_ll_vec_malloc_init(n2 + 1);
  for( i = n1 ; i <= n2 ; i++ ) {
    fld_ll_vec_random(s,i);
    fld_ll_berlekamp_massey(lambda,&nlambda,s,i);
    fld_ll_print(out,"if e eq 0 then\n"
                     "  S := %v;\n"
                     "  S := ChangeUniverse(S,fld);\n"
                     "  f := P ! (%p);\n"
                     "  T := %v;\n"
                     "  T := ChangeUniverse(T,fld);\n"
                     "  U := LFSRSequence(f,T,%d);\n"
                     "  if not (S eq U) then\n"
                     "    print \"Error\";\n"
                     "    e := 1;\n"
                     "  end if;\n"
                     "end if;\n",
                     s,i,lambda,nlambda,s,nlambda,i);
  }
  fld_ll_vec_free(s,n2 + 1);
  fld_ll_vec_free(lambda,n2 + 1);
}

#define D1 10
#define D2 1000

int main(void) {
  srand(time(0));
  fld_ring_init();
  fld_ll_print(stdout,"clear;\n"
                      "%s;\n"
                      "P<X> := PolynomialRing(fld);\n"
                      "e := 0;\n\n",fld_magma_decl);
  mtest_berlekamp_massey(stdout,D1,D2);
  fld_ll_print(stdout,"if e eq 0 then\n"
                      "  print \"ALL OK\";\n"
                      "end if;\n\n");
  return 0;
}

