/*
 * This file is part of decoding
 * Copyright 2012  Guillaume Quintin, INRIA
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#define _POSIX_C_SOURCE 1

#include <time.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <decoding/decoding.h>

#define RING_NAME gf2n
#define DEGREE 16
#include <decoding/rings/mpfq_gf2n_wrapper.c>
#include <decoding/algos.c>

void test_random(FILE *out,int n1,int n2) {
  int i;
  gf2n a;
  gf2n_init(a);
  for( i = n1 ; i <= n2 ; i++ ) {
    gf2n_random(a);
    gf2n_ll_print(out,"a = %e\n",a);
  }
  gf2n_clear(a);
}

void test_next(FILE *out,int n) {
  int i;
  gf2n a;
  gf2n_init(a);
  for( i = 0 ; i <= n ; i++ ) {
    gf2n_ll_print(out,"a = %e\n",a);
    gf2n_next(a,a);
  }
  gf2n_clear(a);
}

void test_binomial(int n) {
  int i,j,bin_mod2;
  gf2n a,b;
  mpz_t bin;

  mpz_init(bin);
  gf2n_init(a);
  gf2n_init(b);
  for( i = 0 ; i < n ; i++ ) {
    for( j = 0 ; j < n ; j++ ) {
      gf2n_ll_binomial(a,i,j);
      mpz_bin_uiui(bin,i,j);
      bin_mod2 = (int)mpz_fdiv_ui(bin,(unsigned long)2) & 1;
      gf2n_castint(b,bin_mod2);
      if ( gf2n_cmp(a,b) ) {
        fprintf(stderr,"Error: Binomial(%d,%d) mod 2 = %d\n",i,j,bin_mod2);
        exit(-1);
      }
    }
  }
  gf2n_clear(b);
  gf2n_clear(a);
  mpz_clear(bin);
}

void test_sse(FILE *out,int n) {
  int i;
  gf2n *a,*b,*c;
  a = gf2n_ll_vec_malloc_init(n);
  b = gf2n_ll_vec_malloc_init(n);
  c = gf2n_ll_vec_malloc_init(n);
  for( i = 0 ; i < n ; i++ ) {
    gf2n_add(c[i],a[i],b[i]);
  }
  gf2n_ll_print(out,"%v\n",c,n);
  gf2n_ll_vec_free(a,n);
  gf2n_ll_vec_free(b,n);
  gf2n_ll_vec_free(c,n);
}

int main(void) {
  ll_random_seed(time(0));
  gf2n_ring_init();
  /* test_random(stdout,0,20);
  test_next(stdout,20);
  test_binomial(100); */
  test_sse(stdout,1000);
  return 0;
}

