/*
 * This file is part of decoding
 * Copyright 2012  Guillaume Quintin, INRIA
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#define _POSIX_C_SOURCE 1

#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <decoding/decoding.h>

#define RING_NAME fld
#define DEGREE 7
#include <decoding/rings/mpfq_gf2n_wrapper.c>
/*
#define PRIME 67
#include <decoding/rings/gfp_word.c>
*/
#include <decoding/algos.c>

void vec_print_list(FILE *out,fld_vec m,fld_vec *a,int na) {
  int i;
  for( i = 0 ; i < na ; i++ ) {
    fld_vec_print(out,a[i]);
    fprintf(stderr," at distance %d\n",fld_vec_hamming_distance(m,a[i]));
    /* fputc('\n',out); */
  }
}

void die(fld_vec m,fld_vec *a,int na) {
  fprintf(stderr,"Error: cannot find message:\n");
  fld_vec_print(stderr,m); fputc('\n',stderr);
  if ( na > 0 ) {
    fprintf(stderr,"List returned:\n");
    vec_print_list(stderr,m,a,na);
  }
  exit(-1);
}

void sudan(int n,int k,int percent,int max) {
  int i,j,rad,tau,nc;
  fld_vec m,y,e,*c;
  fld_rs_code rs;

  fld_vec_inits(m,y,e,0);
  fld_rs_code_init_with_support(rs,n,k);
  rad = sudan_max_errors(n,k);
  tau = (int)((double)percent * (double)rad / 100.0);
  printf("Sudan algorithm (x%d):\n"
         "      n = %d,\n"
         "      k = %d,\n"
         "    tau = %d, (%d%% of errors, max is %d).\n",
         max,n,k,tau,percent,rad);
  for( i = 0 ; i < max ; i++ ) {
    fld_rs_code_random_codeword(m,rs);
    fld_vec_random_hamming_weight(e,n,tau);
    fld_vec_add(y,m,e);
    nc = fld_rs_code_sudan_koetter(&c,rs,y,tau);
    if ( nc == 0 ) die(m,c,0);
    for( j = 0 ; j < nc ; j++ ) {
      if ( fld_vec_cmp(c[j],m) == 0 ) goto sudan_good;
    }
    die(m,c,nc);
sudan_good:;
    fld_vec_ptr_clear(c,nc);
  }
  fld_rs_code_clear(rs);
  fld_vec_clears(m,y,e,0);
  fputs("OK\n",stdout);
}

void guruswami_sudan(int n,int k,int percent,int max) {
  int i,j,rad,tau,nc,mult;
  
  fld_vec m,y,e,*c;
  fld_rs_code rs;

  fld_vec_inits(m,y,e,0);
  fld_rs_code_init_with_support(rs,n,k);
  rad = guruswami_sudan_max_errors(n,k);
  tau = (int)((double)percent * (double)rad / 100.0);
  mult = guruswami_sudan_mult(n,k,tau);
  printf("Guruswami-Sudan algorithm (x%d):\n"
         "      n = %d,\n"
         "      k = %d,\n"
         "    tau = %d, (%d%% of errors, max is %d).\n"
         "   mult = %d\n",
         max,n,k,tau,percent,rad,mult);
  for( i = 0 ; i < max ; i++ ) {
    fld_rs_code_random_codeword(m,rs);
    fld_vec_random_hamming_weight(e,n,tau);
    fld_vec_add(y,m,e);
    nc = fld_rs_code_guruswami_sudan_koetter(&c,rs,y,tau);
    if ( nc == 0 ) die(m,c,0);
    for( j = 0 ; j < nc ; j++ ) {
      if ( fld_vec_cmp(c[j],m) == 0 ) goto guruswami_sudan_good;
    }
    die(m,c,nc);
guruswami_sudan_good:;
    fld_vec_ptr_clear(c,nc);
  }
  fld_rs_code_clear(rs);
  fld_vec_clears(m,y,e,0);
  fputs("OK\n",stdout);
}

int main(void) {
  srand(time(0));
  fld_ring_init();
  sudan(66,16,100,100);
  guruswami_sudan(66,16,95,10);
  return 0;
}

