/*
 * This file is part of decoding
 * Copyright 2012  Guillaume Quintin, INRIA
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#define _POSIX_C_SOURCE 1

#define VERSION "guruswami-sudan version 0.1"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <decoding/decoding.h>

void print_version(FILE *out) {
  fputs(VERSION "\n",out);
}

void print_help(FILE *out) {
  print_version(out);
  fputs("usage: guruswami-sudan n k [--all | --list-size | --max-errors |"
                                  " --errors e | --max-mult | --mult m]\n"
        "  n   The length of the code.\n"
        "  k   The dimension of the code.\n"
        "  --all\n"
        "      Print all the information concerning the code "
                                   "and the Guruswami-Sudan\n"
        "      algorithm.\n"
        "  --max-errors\n"
        "      Print the maximum number of errors that can be"
                                                " corrected\n"
        "      with the Sudan algorithm.\n",out);
  fputs("  --list-size\n"
        "      Print the list size for the Sudan algorithm.\n"
        "  --errors e\n"
        "      Indicates that one wants to correct 'e' errors.\n"
        "  --max-mult\n"
        "      Print the maximum multiplicty needed.\n"
        "  --mult m\n"
        "      Indicated that one wants to use multiplicity 'm'.\n",out);
}

int main(int argc,char **argv) {
  int i;
  int n,k,tau,L,d,t,tau_max,m,m_max;
  if ( argc <= 3 ) {
    print_help(stderr);
    exit(-1);
  }
  n = atoi(argv[1]);
  k = atoi(argv[2]);
  d = n - k + 1;
  t = (n - k) / 2;
  tau_max = guruswami_sudan_max_errors(n,k);
  tau = tau_max;
  m_max = guruswami_sudan_mult(n,k,tau_max);
  m = m_max;
  for ( i = 3 ; i < argc ; i++ ) {
    L = guruswami_sudan_list_size(n,k,tau,m);
    if ( !strcmp(argv[i],"--all") ) {
      printf("Reed-Solomon code with parameters [%d,%d,%d]\n"
             "Unique decoding radius: %d\n"
             "Guruswami-Sudan algorithm:\n"
             "         max-errors: %d\n"
             "          list-size: %d\n"
             "       multiplicity: %d\n",n,k,d,t,tau,L,m);
    }
    else if ( !strcmp(argv[i],"--max-errors") ) {
      printf("%d\n",tau);
    }
    else if ( !strcmp(argv[i],"--list-size") ) {
      printf("%d\n",L);
    }
    else if ( !strcmp(argv[i],"--errors") ) {
      if ( (i + 1) >= argc ) {
        fprintf(stderr,"guruswami-sudan: --errors: missing argument.\n");
        print_help(stderr);
        exit(-1);
      }
      i++;
      tau = atoi(argv[i]);
      if ( tau > tau_max ) tau = tau_max;
      m = guruswami_sudan_mult(n,k,tau);
    }
    else if ( !strcmp(argv[i],"--mult") ) {
      if ( (i + 1) >= argc ) {
        fprintf(stderr,"guruswami-sudan: --mult: missing argument.\n");
        print_help(stderr);
        exit(-1);
      }
      i++;
      m = atoi(argv[i]);
      if ( m > m_max ) m = m_max;
      tau = guruswami_sudan_errors(n,k,m);
    }
    else if ( !strcmp(argv[i],"--max-mult") ) {
      printf("%d\n",m);
    }
    else {
      fprintf(stderr,"guruswami-sudan: %s: unknown option.\n",argv[i]);
      print_help(stderr);
      return -1;
    }
  }
  return 0;
}

